require 'soar_idm/soar_idm'
require 'json'

module Soar
  module Idr
    ##
    # Identity Registry used by {SoarSc::Authorization::Policy::ConfigurationServiceManager}[https://gitlab.host-h.net/policies/configuration_service_manager].
    class ConfigurationServiceManager < SoarIdm::IdmApi

      @@attributes = {
        "configuration_consumer" => {
          "configuration_identifiers" => ['*']
        },
        "configuration_publisher" => {
          "configuration_identifiers" => ['*']
        }
      }

      @@roles = ['configuration_publisher', 'configuration_consumer']

      ##
      # roles
      # @return [Array] Accessor for roles class variable
      def self.roles
        @@roles
      end

      ## 
      # attributes
      # @return [Hash] Accessor for attributes class variable
      def self.attributes
        @@attributes
      end

      ##
      # calculate_roles
      # @param identity [String]
      # @return [Array] List of roles
      def calculate_roles(identity)
        return [@@roles[0]] if publisher?(identity)
        return [@@roles[1]] if consumer?(identity)
        return [@@roles[0], @@roles[1]]
      end

      ## 
      # calculate_attributes
      # @param identity [String]
      # @param role [String]
      # @return [Hash] A hash of attributes 
      def calculate_attributes(identity, role)
        return {
          role => @@attributes[role]
        }
      end 

      ##
      # calculate_all_attributes
      # @param identity [String]
      # @return [Hash] Hash of attributes keyed by role
      def calculate_all_attributes(identity)
        attributes = {}
        roles = calculate_roles(identity)
        roles.each do |role|
          attributes.merge!(calculate_attributes(identity, role))
        end
        attributes
      end

      ##
      # calculate_identities
      # @param identity [String]
      # @return [Array]
      def calculate_identities(entity_identifier)
        hetzner_email?(entity_identifier) ? [entity_identifier] : []
      end

      private

      def hetzner_email?(identity) 
        identity.end_with?('@hetzner.co.za')
      end

      def publisher?(identity)
        local = get_local(identity)
        /\+publisher\z/.match(local)
      end

      def consumer?(identity)
        local = get_local(identity)
        /\+consumer\z/.match(local)
      end

      def get_local(email)
        email.split('@')[0]
      end

    end
  end
end
